<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml"/>

	<xsl:variable name="yscale" select="200"/>
   	<xsl:variable name="xscale" select="300"/>
   	<xsl:variable name="offset" select="20"/>
   	
	<xsl:variable name="max" select="/memory/max"/>
	<xsl:variable name="steps" select="count(/memory//stat)"/>
	<xsl:variable name="xstep" select="$xscale div $steps"/>
   	<xsl:variable name="xb" select="$offset"/>
   	<xsl:variable name="yb" select="$yscale+$offset"/>

    <xsl:template match="/memory">
    <div style="margin: 10px;"><script type="image/svg+xml">
    	<svg
   xmlns:dc="http://purl.org/dc/elements/1.1/"
   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
   xmlns="http://www.w3.org/2000/svg"
   xmlns:xlink="http://www.w3.org/1999/xlink"
   width="{$xscale+2*$offset+70+10}"
   height="{$yscale+2*$offset+10}"
   version="1.0"
   >
   
  
  <g style="font-family:helvetica; font-style:normal; font-weight:bold; font-size:12px;"
     id="layer1">
     <!--
       *********
       DRAW FRAME / AXES
       *********
       -->
    <rect
       style="fill:white;stroke:#bdbdbd;stroke-width:1.23824191;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1"
       id="frame"
       width="{$xscale+2*$offset+70}"
       height="{$yscale+2*$offset}"
       x="0"
       y="0" />
	<text
		class="labeltext"
		x="0"
		y="{$offset - 5}"
		id="ytitle">
			<tspan><xsl:value-of select="round($max div 1024)"/> mB</tspan>
	</text>
	<text
		class="labeltext"
		x="{$offset+$xscale}"
		y="{$offset + $yscale + 12}"
		id="xtitle">
			<tspan><xsl:value-of select="/memory/totalDurationMinutes"/> mins</tspan>
	</text>
       <!--
       *********
       DRAW CHARTS
       *********
       -->
       <path
       class="allocfill" id="alloc" style="fill: #ebb; stroke: none;">
       <xsl:attribute name="d">
       		<xsl:call-template name="fillcurve">
       			<xsl:with-param name="element" select="'alloc'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>
       <path
       class="allocfill" id="alloc" style="fill: none; stroke:#cd0000; stroke-width: 2; stroke-linejoin:round; stroke-linecap:round;">
       <xsl:attribute name="d">
       		<xsl:call-template name="drawcurve">
       			<xsl:with-param name="element" select="'alloc'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>
       
       <path class="regularfill peakfill" id="peak" style="fill:#b9d; stroke: none;">
       <xsl:attribute name="d">
       		<xsl:call-template name="fillcurve">
       			<xsl:with-param name="element" select="'peak'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>
       <path class="regularfill peakfill"  id="peak" style="fill: none; stroke:#425; stroke-width: 2; stroke-linejoin:round; stroke-linecap:round;">
       <xsl:attribute name="d">
       		<xsl:call-template name="drawcurve">
       			<xsl:with-param name="element" select="'peak'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>
       
       <path class="baselinefill" id="baseline" style="fill: #7b9; stroke: none;" >
       <xsl:attribute name="d">
       		<xsl:call-template name="fillcurve">
       			<xsl:with-param name="element" select="'baseline'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>
       <path class="baselinefill" id="baseline" style="fill: none; stroke:#060; stroke-width: 2; stroke-linejoin:round; stroke-linecap:round;" >
       <xsl:attribute name="d">
       		<xsl:call-template name="drawcurve">
       			<xsl:with-param name="element" select="'baseline'"/>
       		</xsl:call-template>
       </xsl:attribute>
       </path>

    <path
       style="fill:none;fill-rule:evenodd;stroke:#000000;stroke-width:2px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
       d="M {$offset},{$offset - 4} L {$offset},{$offset+$yscale} L {$offset+$xscale +4}, {$offset+$yscale}"
       id="axes" />

       
       <!--
       *********
       DRAW TITLE/LEGEND
       *********
       -->
       <text
		class="labeltext"
	       x="{$offset+$xscale+10}"
       		y="20"
	       id="title">
		<tspan id="tspan607">Heap Usage</tspan>
		</text>
       <text
		class="labeltext"
	       x="{$offset+$xscale+10}"
       		y="35"
	       id="title">
		<tspan id="tspan607">(Old Gen)</tspan>
		</text>
		
		<rect
       style="stroke: #cd0000; fill: #ebb;  stroke-width: 2;"
       id="frame"
       width="10"
       height="10"
       x="{$offset+$xscale+10}"
       y="50" />
       <text
			class="labeltext"
	       x="{$offset+$xscale+25}"
       		y="60"
	       id="title">
		<tspan id="tspan607">Committed</tspan>
	   </text>
	   
       <rect
       style="fill:#b9d; fill-opacity:1; stroke:#425;  stroke-width: 2;"
       id="frame"
       width="10"
       height="10"
       x="{$offset+$xscale+10}"
       y="70" />
       <text
		class="labeltext"
	       x="{$offset+$xscale+25}"
       		y="80"
	       id="title">
		<tspan id="tspan607">Peak</tspan>
	   </text>
       
       <rect
       style="stroke: #060; fill: #7b9;  stroke-width: 2;"
       id="frame"
       width="10"
       height="10"
       x="{$offset+$xscale+10}"
       y="90" />
       <text
		class="labeltext"
	       x="{$offset+$xscale+25}"
       		y="100"
	       id="title">
		<tspan id="tspan607">Baseline</tspan>
	   </text>
  </g>
</svg></script>
</div>
    </xsl:template>
    
    <xsl:template name="fillcurve">
    	<xsl:param name="element"/>
    	M <xsl:value-of select="$xb"/>, <xsl:value-of select="$yb"/>
       	<xsl:for-each select="/memory//stat">
       	<xsl:variable name="e" select="./*[name()=$element]"/>
       	<xsl:if test="position()=1">
       	L <xsl:value-of select="round($xb)"/>, <xsl:value-of select="round($yb - (($e div $max) * $yscale ))"/>
       	</xsl:if>
       	L <xsl:value-of select="round($xb+(position()*$xstep))"/>, <xsl:value-of select="round($yb - (($e div $max) * $yscale ))"/>
       	<xsl:if test="position()=last()">
       	L <xsl:value-of select="round($xb+(position()*$xstep))"/>, <xsl:value-of select="round($yb)"/>
       	</xsl:if>
       	</xsl:for-each>
       	L <xsl:value-of select="round($xb)"/>, <xsl:value-of select="round($yb)"/> z
    </xsl:template>
    
    <xsl:template name="drawcurve">
    	<xsl:param name="element"/>
       	<xsl:for-each select="/memory//stat">
       	<xsl:variable name="e" select="./*[name()=$element]"/>
       	<xsl:if test="position()=1">
       	M <xsl:value-of select="round($xb)"/>, <xsl:value-of select="round($yb - (($e div $max) * $yscale ))"/>
       	</xsl:if>
       	L <xsl:value-of select="round($xb+(position()*$xstep))"/>, <xsl:value-of select="round($yb - (($e div $max) * $yscale ))"/>
       	</xsl:for-each>
    </xsl:template>
    

</xsl:stylesheet>