package tools.logging;

import java.util.HashSet;

import org.netkernel.layer0.logging.LogManager;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class EditLoggingLevelsAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode param = (IHDSNode)aContext.source("httpRequest:/params",IHDSNode.class);
		HashSet<String> onLevels = new HashSet<String>();
		if (param.getChildren().length!=0)
		{	for (IHDSNode n : param.getChildren())
			{	onLevels.add(n.getName());
			}
			IHDSNode logs=aContext.source("netkernel:/logLevels",IHDSNode.class);
			HDSBuilder b = new HDSBuilder();
			b.pushNode("logs");
			for (IHDSNode logNode:logs.getNodes("/logs/log"))
			{
				b.pushNode("log");
				String id=logNode.getFirstValue("id").toString();
				b.addNode("id",id);
				b.addNode("name",logNode.getFirstValue("name"));
				b.pushNode("levels");
				for (String levelString: LogManager.sLevelNameLookup.keySet())
				{
					String paramName=id+"-"+levelString;
					b.addNode(levelString, onLevels.contains(paramName));
				}
				b.popNode();
				b.popNode();
			}
			aContext.sink("netkernel:/logLevels", b.getRoot());
		}
			
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/logging/styleLoggingLevels.xsl");
		req.addArgument("operand","netkernel:/logLevels");
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse respOut=aContext.createResponseFrom(resp);
		respOut.setHeader("WrappedControlPanel", true);
	}
}
