package tools.requesttrace;

import java.io.IOException;
import java.io.StringReader;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.nkf.INKFKernelContext;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFRequestImpl;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.nkf.impl.NKFTransportContextImpl;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.layer0.util.SpaceClassLoader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.scheduler.IInternalResponseMeta;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IResolution;
import org.netkernel.urii.ISpace;
import org.netkernel.util.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class RequestTraceAccessor extends StandardAccessorImpl
{
	public RequestTraceAccessor()
	{	
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode params=aContext.source("httpRequest:/params",IHDSNode.class);
		HDSBuilder b= new HDSBuilder();
		b.pushNode("resolve");
		b.importChildren(params);
		
		IHDSNode action1=params.getFirstNode("action1");
		if (action1!=null)
		{
		    if (action1.getValue().equals("resolve"))
		    {	process(aContext,params,b);
		    }
		    else if (action1.getValue().equals("execute"))
		    {	execute(aContext,params,b);
		    }
		    else if (action1.getValue().equals("display"))
		    {	display(aContext,params);
		    	return;
		    }
		}
	       
	    INKFRequest req2 = aContext.createRequest("active:xslt");
        req2.addArgument("operator","res:/tools/requesttrace/styleTrace.xsl");
        req2.addArgumentByValue("param",b.getRoot() );
	    req2.addArgument("operand","active:moduleListDoc");
        INKFResponseReadOnly resp=aContext.issueRequestForResponse(req2);
	    INKFResponse respOut=aContext.createResponseFrom(resp);
	    respOut.setHeader("WrappedControlPanel",true);
	    respOut.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	    respOut.setHeader("httpResponse:/header/Cache-Control","no-cache");
	}
	
	private INKFRequest createRequest(INKFRequestContext aContext, IHDSNode params, IRepDeployedModules mods) throws Exception
	{
		ISpace space=getSpaceFromParams(params, mods);
        String uri=(String)params.getFirstValue("uri");
        
        //INKFKernelContext kc=aContext.getKernelContext();
        //IRequest parentReq=kc.getThisKernelRequest();
        //IRequestScopeLevel parentScope=parentReq.getRequestScope();
        INKFRequest req=null;
        if(uri.indexOf("<")>=0 && uri.indexOf(">")>0)
        {	
        	Document d=org.netkernel.xml.util.XMLUtils.parse(new StringReader(uri));
			RequestBuilder rb=new RequestBuilder(d,aContext.getKernelContext().getKernel().getLogger());
			ClassLoader cl=space.getClassLoader();
			if (cl==null)
			{	cl=Thread.currentThread().getContextClassLoader();
			}
			req=rb.buildRequest(aContext, null, cl);
		
        }
        else
        {	req=aContext.createRequest(uri);
        }
        int verb=Integer.parseInt(params.getFirstValue("verb").toString());
        req.setVerb(verb);
        IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(space);
        //IRequestScopeLevel scope = RequestScopeLevelImpl.createOrphanedRootScopeLevel(space, parentScope);
        req.setRequestScope(scope);
        return req;
	}
	
	private ISpace getSpaceFromParams(IHDSNode params, IRepDeployedModules mods) throws Exception
	{	String spaceString=(String)params.getFirstValue("space");
		ISpace space=(ISpace)mods.componentForHash(spaceString);
	    if (space==null)
	    {	throw new Exception("Space not found");
	    }
	    return space;
	}
	
	private void process(INKFRequestContext aContext, IHDSNode params, HDSBuilder b) throws Exception
	{
	    IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
        INKFRequest req=createRequest(aContext,params,mods);
        IRequest toResolve=((NKFRequestImpl)req).getKernelRequest();
        
        TraceDebugCapture tdc = new TraceDebugCapture();
        IRequest parentReq=aContext.getKernelContext().getThisKernelRequest();
		IRequest resolveReq = RequestFactory.createResolveRequest(toResolve, null, tdc, parentReq, null);
        IResolution resolution;
        try
        {	resolution = (IResolution)aContext.getKernelContext().issueKernelRequest(resolveReq).getRepresentation();
        }
        catch (Exception e)
        {	resolution = null;
        }
        
        b.pushNode("resolution");        
        IHDSNode resolve=aContext.transrept(tdc, IHDSNode.class);
        b.importChildren(resolve);
        //b.popNode();
        
        if (resolution!=null)
        {	
        	IEndpoint endpoint = resolution.getEndpoint();
        	String endpointPhysicalHash=mods.hashForComponent(endpoint);
        	
        	b.addNode("class", endpoint.getClass().getName());
        	b.addNode("toString", endpoint.toString());
        	b.addNode("physicalHash", endpointPhysicalHash);

        	IIdentifier lii=resolution.getEndpointIdentifier();
        	if (lii!=null)
        	{
	        	String logicalIdentifier=lii.toString();
	        	ISpace space=resolution.getScope().getSpace();
	        	String endpointLogicalHash=mods.hashForComponent(space)+"/"+logicalIdentifier;
	        	b.addNode("id",logicalIdentifier);
	        	b.addNode("logicalHash",endpointLogicalHash);
        	}
        	
        	
        	IRequestScopeLevel level = resolution.getScope();
        	List<ISpace> spaces=new ArrayList<ISpace>();
        	while (level!=null)
        	{	spaces.add(0, level.getSpace());
        		level=level.getParent();
        	}
        	
        	b.pushNode("scope");
        	for (ISpace scopeSpace: spaces)
        	{	b.pushNode("level");
        		b.addNode("space",scopeSpace.toString());
        		String spacelHash=mods.hashForComponent(scopeSpace);
        		b.addNode("hash",spacelHash);
        		b.popNode();
        	}
        	b.popNode();
        }
		
	}
	
	private void execute(INKFRequestContext aContext, IHDSNode params, HDSBuilder b) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
		INKFKernelContext kc=aContext.getKernelContext();
		ISpace space=getSpaceFromParams(params, mods);
		INKFRequestContext tContext=new NKFTransportContextImpl(kc.getKernel(),space,this);
		
		INKFRequest req=createRequest(tContext,params,mods);
        try
        {	INKFResponseReadOnly resp = tContext.issueRequestForResponse(req);
        
	        b.pushNode("response");
	        Object representation=resp.getRepresentation();
	        if (representation!=null)
	        {	b.addNode("repClass", representation.getClass().getName());
		        String repHash=mods.hashForClass(representation.getClass());
		        if (repHash!=null)
		        {	b.addNode("repHash", repHash);
		        	String repName=mods.metadataForHash(repHash).getName();
		        	b.addNode("repName", repName);
		        }
		        b.addNode("repToString", representation.toString());
	        }
	        else
	        {	b.addNode("repClass", "<null>");
	        }
	        
	        IResponseMeta meta=((NKFResponseReadOnlyImpl)resp).getMeta();
			boolean isExpired=meta.getExpiry().isExpired(System.currentTimeMillis());
			b.addNode("isExpired", Boolean.toString(isExpired));
			String expiryString=ExpiryFactory.toString(meta.getExpiry());
			IHDSNode expiryHDS=aContext.transrept(expiryString, IHDSNode.class);
			b.pushNode("expiry");
			b.importChildren(expiryHDS);
			b.popNode();
			
			b.pushNode("usermeta");
			IRequestResponseFields userMeta=meta.getUserMetaData();
			int userMetaSize=userMeta.size();
			for (int j=0; j<userMetaSize; j++ )
			{	String key = userMeta.getKey(j);
				b.pushNode("value",userMeta.getValue(j));
				b.addNode("key",key);
				b.popNode();
			}
			b.popNode();			
			
			if (meta instanceof IInternalResponseMeta)
			{	IInternalResponseMeta im=(IInternalResponseMeta)meta;
				b.addNode("cost", Integer.toString(im.getTotalCost()));
			}
	        
	        b.popNode();
        }
        catch (NKFException e)
        {	throw e;
        }
        
	}
	
	private void display(INKFRequestContext aContext, IHDSNode params) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
		INKFKernelContext kc=aContext.getKernelContext();
		ISpace space=getSpaceFromParams(params, mods);
		INKFRequestContext tContext=new NKFTransportContextImpl(kc.getKernel(),space,this);
		
		INKFRequest req=createRequest(tContext,params,mods);
        INKFResponseReadOnly resp = tContext.issueRequestForResponse(req);
        INKFResponse respOut=aContext.createResponseFrom(resp);
        respOut.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	    respOut.setHeader("httpResponse:/header/Cache-Control","no-cache");
	}

}